//
//  TVOCallDelegate.h
//  TwilioVoice
//
//  Copyright © 2018 Twilio, Inc. All rights reserved.
//

@class TVOCall;

/**
 *  `TVOCallDelegate` provides callbacks when important changes to a `TVOCall` occur.
 */
@protocol TVOCallDelegate <NSObject>

/**
 * @name Required Methods
 */

/**
 * @brief Notifies the delegate that a Call has connected.
 *
 * @param call The `<TVOCall>` that was connected.
 *
 * @see TVOCall
 */
- (void)callDidConnect:(nonnull TVOCall *)call;

/**
 * @brief Notifies the delegate that a Call has failed to connect.
 *
 * @param call The `<TVOCall>` that failed to connect.
 * @param error The `<NSError>` that occurred.
 *
 * @see TVOCall
 */
- (void)call:(nonnull TVOCall *)call didFailToConnectWithError:(nonnull NSError *)error;

/**
 * @brief Notifies the delegate that a Call has disconnected.
 *
 * @param call The `<TVOCall>` that was disconnected.
 * @param error An `NSError` describing why disconnect occurred, or `nil` if the disconnect was expected.
 *
 * @see TVOCall
 */
- (void)call:(nonnull TVOCall *)call didDisconnectWithError:(nullable NSError *)error;

/**
 * @name Optional Methods
 */

@optional
/**
 * @brief Notifies the delegate that the called party is being alerted of a Call.
 *
 * @param call The `<TVOCall>` that the called party is being alerted of.
 *
 * @discussion Introduced a new callback method `[TVOCallDelegate callDidStartRinging:]`. This callback is invoked once
 * before the `[TVOCallDelegate callDidConnect:]` callback when the callee is being alerted of a Call. The duration of
 * the `TVOCallStateRinging` is determined by the `answerOnBridge` flag provided in the `Dial` verb of your TwiML
 * application associated with this client. If the `answerOnBridge` flag is `false`, which is the default, the
 * `[TVOCallDelegate callDidConnect:]` callback will be emitted immediately after
 * `[TVOCallDelegate callDidStartRinging:]`. If the `answerOnBridge` flag is `true` this will cause the Call to emit
 * the `callDidConnect` callback only until the call is answered.
 * See https://www.twilio.com/docs/voice/twiml/dial#answeronbridge for more details on how to use it with the `Dial`
 * TwiML verb. If the twiML response contains a `Say` verb, then the call will emit the
 * `[TVOCallDelegate callDidConnect:]` callback immediately after `[TVOCallDelegate callDidStartRinging:]` is raised,
 * irrespective of the value of `answerOnBridge` being set to `true` or `false`.
 *
 * @see TVOCall
 */
- (void)callDidStartRinging:(nonnull TVOCall *)call;

@end
